package ch.polgrabia.demos.client;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.AsyncResult;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.http.HttpServer;
import io.vertx.core.http.HttpServerOptions;
import io.vertx.core.http.HttpServerRequest;
import io.vertx.core.http.HttpServerResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class WebApiParticle extends AbstractVerticle {
  private static final Logger logger = LoggerFactory.getLogger(WebApiParticle.class);
  private HttpServer server;

  @Override
  public void start() throws Exception {
    HttpServerOptions httpServerOptions = new HttpServerOptions();
    this.server = getVertx().createHttpServer(httpServerOptions);
    this.server.requestHandler(this::handleCommandRequests);
    this.server.listen(8123, "localhost", this::handleListenCommand);
  }

  @Override
  public void stop() throws Exception {
    server.close();
  }

  private void handleCommandRequests(HttpServerRequest httpServerRequest) {
    logger.info("Received command request on 8123");
    HttpServerResponse response = httpServerRequest.response();
    String qParam = httpServerRequest.getParam("q");
    if (qParam == null || qParam.isBlank()) {
      response.setStatusCode(400);
      response.end("bad request");
      return;
    }
    logger.info("Sending binary message: {}", qParam);
    getVertx().eventBus().send(Constants.CLIENT_WEBSOCKET_SENDER_EVENTBUS_KEY, Buffer.buffer(qParam));
    response.setStatusCode(202);
    response.end("Ok");
  }

  private void handleListenCommand(AsyncResult<HttpServer> httpServerAsyncResult) {
    if (httpServerAsyncResult.failed()) {
      logger.warn("Couldn't initialized http command server");
      getVertx().close();
      System.exit(1);
    }

    logger.info("Successfully started websocket client command server");
  }
}
